/*  
 *  LPT Manager
 *  Copyright (C) 2002, 2003  Giovanni Pardini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "functions.h"

#ifdef G_OS_UNIX
#include <sys/io.h>
#endif

#ifdef G_OS_WIN32
#include <windows.h>
#endif

#include <errno.h>
#include <string.h>

#include "lptmanager.h"
#include "support.h"
#include "options.h"
#include "aboutDlg.h"
#include "pixbufs.h"


enum IOFuncType {IOFT_NONE, IOFT_NATIVE, IOFT_DLPORTIO, IOFT_IODLL};

static gboolean IOconfig_initialized = FALSE;

struct
{
	enum    IOFuncType type;
	
	guint8 (*inb)(guint16 port);
	void (*outb)(guint16 port, guint8 value);

} IOconfig;


/* Functions for direct access to I/O ports */

inline guint8 w32_inb(guint16 port)
{
	guint8 value = 0;
	
	#ifdef G_OS_WIN32
	__asm__ __volatile__ ("inb %w1,%b0":"=a"(value):"d"(port), "0"(0));
	#endif
	
	return value;
}


inline void w32_outb(guint16 port, guint8 value)
{
	#ifdef G_OS_WIN32
	__asm__ __volatile__ ("outb %b0,%w1"::"a"(value), "d"(port));
	#endif
}

inline void unix_outb(guint16 port, guint8 value)
{
	#ifdef G_OS_UNIX
	outb(value, port);
	#endif
}


gchar *strIOFuncType(void)
{
	switch (IOconfig.type)
	{
		case IOFT_NONE:
			return _("UNAVAILABLE");
			
		case IOFT_NATIVE:
			return _("NATIVE");
			
		case IOFT_DLPORTIO:
			return _("DLPORTIO");
		
		case IOFT_IODLL:
			return _("IODLL");
	}
	return "";
}



/* returns 0 if parallel port access is permitted (or available) */
gint getPermissions(guint16 address)
{
	
#ifdef G_OS_UNIX
	IOconfig_initialized = TRUE;
	IOconfig.type = IOFT_NATIVE;
	IOconfig.inb  = inb;
	IOconfig.outb = unix_outb;
	
	errno = 0;
	ioperm(address, 3, 1);
	return errno;
#endif
	
	if (IOconfig_initialized)
	{
		if (IOconfig.type == IOFT_NONE)
			return 1;
		else
			return 0;
	}

	IOconfig.type = IOFT_NONE;
	IOconfig_initialized = TRUE;
	


#ifdef G_OS_WIN32

	/* determine if inb/outb functions are available */
	HMODULE lib;
	OSVERSIONINFO osInfo;
	
	/* check windows version */
	osInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&osInfo);
	if (osInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
	{
		IOconfig.type = IOFT_NATIVE;
		IOconfig.inb  = w32_inb;
		IOconfig.outb = w32_outb;
		return 0;
	}
	
	/* this windows version does not allow direct access, trying other
	   methods */
	
	/* try to load IO.DLL */
	lib = LoadLibrary("IO.DLL");
	if (lib != NULL)
	{
		guint16(*IsDriverInstalled)() = (guint16(*)())GetProcAddress(lib, "IsDriverInstalled");
		if (!IsDriverInstalled()) {
			
			g_warning(_("IO.DLL has been loaded successful, but the driver is not installed!\n"));
			
		} else {
			
			IOconfig.inb  = (guint8(*)(guint16))GetProcAddress(lib, "PortIn");
			IOconfig.outb = (void(*)(guint16, guint8))GetProcAddress(lib, "PortOut");
			if (IOconfig.inb != NULL && IOconfig.outb != NULL)
			{
				IOconfig.type = IOFT_IODLL;
				return 0;
			}
		}
	}
	
	/* try to load DLPORTIO.DLL */
	lib = LoadLibrary("DLPORTIO.DLL");
	if (lib != NULL)
	{
		IOconfig.inb  = (guint8(*)(guint16))GetProcAddress(lib,  "DlPortReadPortUchar");
		IOconfig.outb = (void(*)(guint16, guint8))GetProcAddress(lib, "DlPortWritePortUchar");
		if (IOconfig.inb != NULL && IOconfig.outb != NULL)
		{
			IOconfig.type = IOFT_DLPORTIO;
			return 0;
		}
	}

	g_warning(_("I/O functions are not available on this system.\n"));
	return 1;
#endif

}

guint8 inbyte(guint16 address)
{
	if (!options_get_is_valid_addr())
		return 0;

	if (IOconfig.type != IOFT_NONE) {
		return IOconfig.inb(address);
	}
	
	return 0;
}

void outbyte(guint16 address, guint8 val)
{
	if (!options_get_modify() || !options_get_is_valid_addr())
		return;
	
	if (IOconfig.type != IOFT_NONE) {
		IOconfig.outb(address, val);
	}
}

void invertBit(guint16 address, guint8 bitNum)
{
	guint8 val, tmp;

	val = inbyte(address);
	tmp = 0x1;
	tmp <<= bitNum;
	
	if (val & tmp) // the bit is 1?
		val &= ~tmp;
	else
		val |= tmp;
	
	outbyte(address, val);
}


gboolean updateWindow(void)
{
	guint8 val;
	GdkPixbuf *green_on;
	GdkPixbuf *green_off;
	GdkPixbuf *red_on;
	GdkPixbuf *red_off;
	
	GtkWidget *DataPortValueSBtn    = lookup_widget(publicVars.mainWnd, "DataPortValueSBtn");
	GtkWidget *StatusPortValueLbl   = lookup_widget(publicVars.mainWnd, "StatusPortValueLbl");
	GtkWidget *ControlPortValueSBtn = lookup_widget(publicVars.mainWnd, "ControlPortValueSBtn");
	
	gchar     *buf;

	gboolean   showLed;
	
	showLed = options_get_showLed();
	
	if (showLed)
	{
		green_on  = gdk_pixbuf_new_from_inline(sizeof( led_green_on_pixbuf),  led_green_on_pixbuf, FALSE, NULL);
		green_off = gdk_pixbuf_new_from_inline(sizeof(led_green_off_pixbuf), led_green_off_pixbuf, FALSE, NULL);
		red_on    = gdk_pixbuf_new_from_inline(sizeof(   led_red_on_pixbuf),    led_red_on_pixbuf, FALSE, NULL);
		red_off   = gdk_pixbuf_new_from_inline(sizeof(  led_red_off_pixbuf),   led_red_off_pixbuf, FALSE, NULL);
	}
	else
	{
		green_on  = gdk_pixbuf_new_from_inline(sizeof( bit_green_on_pixbuf),  bit_green_on_pixbuf, FALSE, NULL);
		green_off = gdk_pixbuf_new_from_inline(sizeof(bit_green_off_pixbuf), bit_green_off_pixbuf, FALSE, NULL);
		red_on    = gdk_pixbuf_new_from_inline(sizeof(   bit_red_on_pixbuf),    bit_red_on_pixbuf, FALSE, NULL);
		red_off   = gdk_pixbuf_new_from_inline(sizeof(  bit_red_off_pixbuf),   bit_red_off_pixbuf, FALSE, NULL);
	}
			
	val = inbyte(options_get_baseaddr());
	updateBit("imageP9", val, 7, green_on, green_off);
	updateBit("imageP8", val, 6, green_on, green_off);
	updateBit("imageP7", val, 5, green_on, green_off);
	updateBit("imageP6", val, 4, green_on, green_off);
	updateBit("imageP5", val, 3, green_on, green_off);
	updateBit("imageP4", val, 2, green_on, green_off);
	updateBit("imageP3", val, 1, green_on, green_off);
	updateBit("imageP2", val, 0, green_on, green_off);
	if (val != gtk_spin_button_get_value (GTK_SPIN_BUTTON(DataPortValueSBtn)))
		gtk_spin_button_set_value (GTK_SPIN_BUTTON(DataPortValueSBtn), val);
	
	
	val = inbyte(options_get_baseaddr() + 1);
	
	updateBit2("imageP11", val, 7, red_on, red_off, showLed);
	updateBit("imageP10", val, 6, red_on, red_off);
	updateBit("imageP12", val, 5, red_on, red_off);
	updateBit("imageP13", val, 4, red_on, red_off);
	updateBit("imageP15", val, 3, red_on, red_off);

	buf = g_strdup_printf("%d", (guint8)(val & 0xF8));
	if (strcmp(buf, gtk_label_get_text(GTK_LABEL(StatusPortValueLbl))) != 0)
		gtk_label_set_text(GTK_LABEL(StatusPortValueLbl), buf);
	g_free(buf);
	
	
	val = inbyte(options_get_baseaddr() + 2);
	updateBit("imageE",   val, 4, green_on, green_off);
	updateBit2("imageP17", val, 3, green_on, green_off, showLed);
	updateBit("imageP16", val, 2, green_on, green_off);
	updateBit2("imageP14", val, 1, green_on, green_off, showLed);
	updateBit2("imageP1",  val, 0, green_on, green_off, showLed);
	
	if ((val & 0x1F) != gtk_spin_button_get_value (GTK_SPIN_BUTTON(ControlPortValueSBtn)));
		gtk_spin_button_set_value (GTK_SPIN_BUTTON(ControlPortValueSBtn), val & 0x1F);
	
	return TRUE;
}

gboolean updateBit(gchar *WidgetName, guint8 value, guint8 bit, GdkPixbuf *pix_on, GdkPixbuf *pix_off)
{
	GtkWidget *image;
	gboolean on;
	
	image = lookup_widget(publicVars.mainWnd, WidgetName);
	on = value & (1 << bit);
	if (on)
		gtk_image_set_from_pixbuf(GTK_IMAGE(image), pix_on);
	else
		gtk_image_set_from_pixbuf(GTK_IMAGE(image), pix_off);
	
	return on;
}

gboolean updateBit2(gchar *WidgetName, guint8 value, guint8 bit, GdkPixbuf *pix_on, GdkPixbuf *pix_off, gboolean swapImages)
{
	return updateBit(WidgetName, value, bit, swapImages ? pix_off : pix_on, swapImages ? pix_on : pix_off);
}

void images_set_sensitive(gboolean sensitive)
{
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP9"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP8"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP7"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP6"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP5"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP4"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP3"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP2"), sensitive);
	
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP11"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP10"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP12"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP13"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP15"), sensitive);
	
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,   "imageE"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd, "imageP17"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd, "imageP16"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd, "imageP14"), sensitive);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd,  "imageP1"), sensitive);
}

void showErrorDlg(GtkWindow *transient, gchar *text)
{
	GtkWidget *errorDlg;
	errorDlg = gtk_message_dialog_new(transient,
	                                  GTK_DIALOG_DESTROY_WITH_PARENT,
	                                  GTK_MESSAGE_ERROR,
	                                  GTK_BUTTONS_OK,
	                                  text);
	gtk_dialog_run (GTK_DIALOG (errorDlg));
	gtk_widget_destroy (errorDlg);
}
