/*  
 *  LPT Manager
 *  Copyright (C) 2002, 2003  Giovanni Pardini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "options.h"

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#ifdef G_OS_WIN32
#include <windows.h>
#endif

#include "lptmanager.h"
#include "functions.h"
#include "support.h"
#include "mainWnd_cb.h"


/* PRIVATE FUNCTIONS DECLARATIONS */

#ifdef G_OS_UNIX
gboolean chdir_home(void);
#endif

/* PRIVATE FUNCTIONS DEFINITIONS */

#ifdef G_OS_UNIX
gboolean chdir_home(void)
{
	gchar *homedir;
	
	homedir = getenv("HOME");
	
	if (homedir == NULL)
		return FALSE;
	if (chdir(homedir) != 0)
		return FALSE;

	return TRUE;
}
#endif


/* PUBLIC FUNCTIONS DEFINITIONS */

void loadOptions(void)
{


	struct options_t tmp;
	gint ret;
#ifdef G_OS_WIN32
	HKEY appKey;
#endif

	tmp.baseaddr = 0x378;
	tmp.timeout  = 100;
	tmp.monitor  = TRUE;
	tmp.modify   = TRUE;
	tmp.showLed  = TRUE;
	

#ifdef G_OS_UNIX

	if (chdir_home())
	{
		FILE *f;
		
		f = fopen(CONFIGURATION_FILE, "r");
		if (f != NULL)
		{
			char prop[10], val[10];
			while (fscanf(f, "%10s %10s \n", prop, val) == 2)
			{
				if (strcmp(prop, PACKAGE) == 0)
				{
					if (strcmp(val, VERSION) != 0)
						g_warning(_("Version of configuration file is unknown: '%s'.\n"), val);
				}
				else if (strcmp(prop, "baseaddr") == 0)
					tmp.baseaddr = strtol(val, NULL, 16);
				else if (strcmp(prop, "timeout") == 0)
					tmp.timeout = strtol(val, NULL, 10);
				else if (strcmp(prop, "monitor") == 0)
					tmp.monitor = (strcmp(val, "ON") == 0);
				else if (strcmp(prop, "modify") == 0)
					tmp.modify = (strcmp(val, "ON") == 0);
				else if (strcmp(prop, "showLed") == 0)
					tmp.showLed = (strcmp(val, "ON") == 0);
			}
			fclose(f);
		}
		else
			g_warning(_("Could not open configuration file. Using default configuration.\n"));
	}
	else
		g_warning(_("Could not access home directory. Using default configuration.\n"));
#endif

#ifdef G_OS_WIN32

	if (RegOpenKeyEx(HKEY_CURRENT_USER, REGISTRY_KEY, 0, KEY_READ, &appKey) == ERROR_SUCCESS)
	{
		LONG ret;
		int i = 0;
		LPSTR value;
		DWORD valueSize = 30;
		PTBYTE data;
		DWORD dataSize = 30;
		
		value = g_malloc(sizeof(TCHAR) * valueSize);
		data  = g_malloc(sizeof(BYTE ) * dataSize );
		
		do
		{
			valueSize = 30;
			dataSize = 30;
			
			ret = RegEnumValue(appKey, i, value, &valueSize, NULL, NULL, data, &dataSize);
			if (ret == ERROR_SUCCESS)
			{
				if (strcmp(value, "baseaddr") == 0)
					tmp.baseaddr = *(PDWORD)data;
				else if (strcmp(value,  "timeout") == 0)
					tmp.timeout =  *(PDWORD)data;
				else if (strcmp(value,  "monitor") == 0)
					tmp.monitor = (*(PDWORD)data != 0);
				else if (strcmp(value,   "modify") == 0)
					tmp.modify =  (*(PDWORD)data != 0);
				else if (strcmp(value,  "showLed") == 0)
					tmp.showLed = (*(PDWORD)data != 0);
			}
			else if (ret != ERROR_NO_MORE_ITEMS)
			{
				gchar *buf;
				buf = g_win32_error_message(ret);
				g_warning(_("RegEnumValue failed with error %ld (%s). Value skipped.\n"), ret, buf);
				g_free(buf);
			}
			
			i++;
		}
		while (ret != ERROR_NO_MORE_ITEMS);
		
		g_free(value);
		g_free(data);
		
		RegCloseKey(appKey);
	}
	else
	{
		g_warning(_("Could not open registry key. Using default configuration.\n"));
	}

#endif

	options_set_timeout(tmp.timeout);
	options_set_monitor(tmp.monitor);
	options_set_modify(tmp.modify);
	options_set_showLed(tmp.showLed);
	
	ret = options_set_baseaddr(tmp.baseaddr);
	
	if (ret != 0)
	{
		GtkWidget *vbox2 = lookup_widget(publicVars.mainWnd, "vbox2");
		gchar *buf;

		gtk_widget_set_sensitive(vbox2, FALSE);		

		/* Show error message */
#ifdef G_OS_UNIX
		buf = g_strdup_printf(_("Could not get I/O permissions.\n\nDetails: %s\n(Note that you must be root to run this program!)"), g_strerror(ret));
#endif
#ifdef G_OS_WIN32
		buf = g_strdup_printf(_("Could not find any suitable system support for port I/O.\n\nYou need to install \"IO.DLL\" or \"DriverLINX Port I/O Driver\" to run this program with this Windows version."));
#endif
							   
		showErrorDlg(GTK_WINDOW(publicVars.mainWnd), buf);
		g_free(buf);
	}
}

void saveOptions(void)
#ifdef G_OS_UNIX
{
	if (chdir_home())
	{
		FILE *f;
		
		f = fopen(CONFIGURATION_FILE, "w");
		if (f != NULL)
		{
			fprintf(f, "%s %s\n",         PACKAGE, VERSION                     );
			fprintf(f, "baseaddr 0x%X\n", options_get_baseaddr()               );
			fprintf(f, "timeout %d\n",    options_get_timeout ()               );
			fprintf(f, "monitor %s\n",    options_get_monitor () ? "ON" : "OFF");
			fprintf(f, "modify %s\n",     options_get_modify  () ? "ON" : "OFF");
			fprintf(f, "showLed %s\n",    options_get_showLed () ? "ON" : "OFF");	
			
			fclose(f);
		}
		else
			g_warning(_("Could not open configuration file. Configuration not saved.\n"));
	}
	else
		g_warning(_("Could not access home directory. Configuration not saved\n"));
}
#endif

#ifdef G_OS_WIN32

{
	HKEY appKey;
	
	if (RegCreateKeyEx(HKEY_CURRENT_USER, REGISTRY_KEY, 0, 
					   NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &appKey,
					   NULL) == ERROR_SUCCESS)
	{
		DWORD val;
		
		val = options_get_baseaddr();
		RegSetValueEx(appKey, "baseaddr", 0, REG_DWORD, (PBYTE)&val, sizeof(DWORD));
		val = options_get_timeout();
		RegSetValueEx(appKey, "timeout",  0, REG_DWORD, (PBYTE)&val, sizeof(DWORD));
		val = options_get_monitor();
		RegSetValueEx(appKey, "monitor",  0, REG_DWORD, (PBYTE)&val, sizeof(DWORD));
		val = options_get_modify();
		RegSetValueEx(appKey, "modify",   0, REG_DWORD, (PBYTE)&val, sizeof(DWORD));
		val = options_get_showLed();
		RegSetValueEx(appKey, "showLed",  0, REG_DWORD, (PBYTE)&val, sizeof(DWORD));
		
		RegCloseKey(appKey);
	}
	else
	{
		g_warning(_("Could not open registry key. Configuration not saved.\n"));
	}
}
#endif

void OptionsCleanup(void)
{
	options_set_monitor(FALSE);
}

gboolean options_get_is_valid_addr(void)
{
	return options.is_valid_addr;
}

gint options_set_baseaddr(guint32 baseaddr)
{
	GtkWidget *addressBase0Lbl = lookup_widget(publicVars.mainWnd, "addressBase0Lbl");
	GtkWidget *addressBase1Lbl = lookup_widget(publicVars.mainWnd, "addressBase1Lbl");
	GtkWidget *addressBase2Lbl = lookup_widget(publicVars.mainWnd, "addressBase2Lbl");
	gchar *buf;
	gboolean modify;
	gint ret;
	
	options.baseaddr = baseaddr;
	
	/* update labels */
	buf = g_strdup_printf(_("Data Port\n<big><b><i>0x%X</i></b></big>"), options.baseaddr);
	gtk_label_set_markup(GTK_LABEL(addressBase0Lbl), buf);
	g_free(buf);
	buf = g_strdup_printf(_("Status Port\n<big><b><i>0x%X</i></b></big>"), options.baseaddr + 1);
	gtk_label_set_markup(GTK_LABEL(addressBase1Lbl), buf);
	g_free(buf);
	buf = g_strdup_printf(_("Control Port\n<big><b><i>0x%X</i></b></big>"), options.baseaddr + 2);
	gtk_label_set_markup(GTK_LABEL(addressBase2Lbl), buf);
	g_free(buf);
	

	if ((ret = getPermissions(options.baseaddr)) != 0)
	{
		// could not get permissions!!!
		options.is_valid_addr = FALSE;
		return ret;
	}
	
	options.is_valid_addr = TRUE;
	
	modify = options_get_modify();
	options_set_modify(TRUE);
	outbyte(options.baseaddr + 1, inbyte(options.baseaddr + 1) & 0xF8);
	outbyte(options.baseaddr + 2, inbyte(options.baseaddr + 2) & 0x1F);
	options_set_modify(modify);

	return 0;
}

guint32 options_get_baseaddr(void)
{
	return options.baseaddr;
}

void options_set_timeout(guint32 timeout)
{
	options.timeout = timeout;
	if (options_get_monitor())
	{
		options_set_monitor(FALSE);
		options_set_monitor(TRUE);
	}
}

guint32 options_get_timeout(void)
{
	return options.timeout;
}

void options_set_monitor(gboolean status)
{
	GtkWidget *showOpt        = lookup_widget(publicVars.mainWnd, "showOpt");
	GtkWidget *monitoringTBtn = lookup_widget(publicVars.mainWnd, "monitoringTBtn");

	gtk_widget_set_sensitive(GTK_WIDGET(showOpt), status);

	g_assert(g_signal_handlers_block_matched(monitoringTBtn, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, &on_monitoringTBtn_toggled, NULL) == 1);

	if (status)
	{
		gtk_button_set_label(GTK_BUTTON(monitoringTBtn), _("ACTIVE"));
		if (publicVars.timeoutId < 0)
			publicVars.timeoutId = gtk_timeout_add(options.timeout, (GSourceFunc)&updateWindow, NULL);
	}
	else
	{
		gtk_button_set_label(GTK_BUTTON(monitoringTBtn), _("INACTIVE"));
		if (publicVars.timeoutId > 0)
			g_source_remove(publicVars.timeoutId);
		publicVars.timeoutId = -1;
	}
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(monitoringTBtn), status);
	
	g_assert(g_signal_handlers_unblock_matched(monitoringTBtn, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, &on_monitoringTBtn_toggled, NULL) == 1);
	
	images_set_sensitive(status);
}

gboolean options_get_monitor(void)
{
	return publicVars.timeoutId > 0 ? TRUE : FALSE;
}

void options_set_modify(gboolean status)
{
	GtkWidget *ModifyTBtn = lookup_widget(publicVars.mainWnd, "ModifyTBtn");
	
	options.modify = status;

	// disable handler for 'toggled' signal
	g_assert(g_signal_handlers_block_matched(ModifyTBtn, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, &on_ModifyTBtn_toggled, NULL) == 1);

	// set button label
	if (status)
		gtk_button_set_label(GTK_BUTTON(ModifyTBtn), _("ACTIVE"));
	else
		gtk_button_set_label(GTK_BUTTON(ModifyTBtn), _("INACTIVE"));

	// set toggle button status
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ModifyTBtn), status);

	// re-enable original handler	
	g_assert(g_signal_handlers_unblock_matched(ModifyTBtn, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, &on_ModifyTBtn_toggled, NULL) == 1);
	
	// set sensitive status of spin buttons
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd, "DataPortValueSBtn"), status);
	gtk_widget_set_sensitive(lookup_widget(publicVars.mainWnd, "ControlPortValueSBtn"), status);
}

gboolean options_get_modify(void)
{
	return options.modify;
}

void options_set_showLed(gboolean status)
{
	GtkWidget *showOpt = lookup_widget(publicVars.mainWnd, "showOpt");
	gtk_option_menu_set_history(GTK_OPTION_MENU(showOpt), status ? 0 : 1);
	options.showLed = status;
	
}

gboolean options_get_showLed(void)
{
	return options.showLed;
}
